@@ -323,7 +323,13 @@ def get_hostvars_from_server(cloud, server, mounts=None):
     return server_vars
 
 
-def obj_to_dict(obj):
+def _add_request_id(obj, request_id):
+    if request_id is not None:
+        obj['x_openstack_request_ids'] = [request_id]
+    return obj
+
+
+def obj_to_dict(obj, request_id=None):
     """ Turn an object with attributes into a dict suitable for serializing.
 
     Some of the things that are returned in OpenStack are objects with
@@ -334,25 +340,33 @@ def obj_to_dict(obj):
     """
     if obj is None:
         return None
-    elif type(obj) == munch.Munch or hasattr(obj, 'mock_add_spec'):
+    elif isinstance(obj, munch.Munch) or hasattr(obj, 'mock_add_spec'):
         # If we obj_to_dict twice, don't fail, just return the munch
         # Also, don't try to modify Mock objects - that way lies madness
         return obj
-    elif type(obj) == dict:
-        return munch.Munch(obj)
     elif hasattr(obj, 'schema') and hasattr(obj, 'validate'):
         # It's a warlock
-        return warlock_to_dict(obj)
+        return _add_request_id(warlock_to_dict(obj), request_id)
+    elif isinstance(obj, dict):
+        # The new request-id tracking spec:
+        # https://specs.openstack.org/openstack/nova-specs/specs/juno/approved/log-request-id-mappings.html
+        # adds a request-ids attribute to returned objects. It does this even
+        # with dicts, which now become dict subclasses. So we want to convert
+        # the dict we get, but we also want it to fall through to object
+        # attribute processing so that we can also get the request_ids
+        # data into our resulting object.
+        instance = munch.Munch(obj)
+    else:
+        instance = munch.Munch()
 
-    instance = munch.Munch()
     for key in dir(obj):
         value = getattr(obj, key)
         if isinstance(value, NON_CALLABLES) and not key.startswith('_'):
             instance[key] = value
-    return instance
+    return _add_request_id(instance, request_id)
 
 
-def obj_list_to_dict(list):
+def obj_list_to_dict(obj_list, request_id=None):
     """Enumerate through lists of objects and return lists of dictonaries.
 
     Some of the objects returned in OpenStack are actually lists of objects,
@@ -360,8 +374,8 @@ def obj_list_to_dict(list):
     the conversion to lists of dictonaries.
     """
     new_list = []
-    for obj in list:
-        new_list.append(obj_to_dict(obj))
+    for obj in obj_list:
+        new_list.append(obj_to_dict(obj, request_id=request_id))
     return new_list
 
 
